require 'fileutils'

# Module that controls the colors used
module Tty extend self

  def black; bold 30; end
  def red; bold 31; end
  def green; bold 32; end
  def yellow; bold 33; end
  def blue; bold 34; end
  def purple; bold 35; end
  def cyan; bold 36; end
  def white; bold 39; end
  def grey; bold 38; end
  def reset; escape 0; end
  def bold n; escape "1;#{n}" end
  def underline n; escape "4;#{n}" end
  def escape n; "\033[#{n}m" if STDOUT.tty? end

end

# Core Module
module DQS

	class CheckDependencies

		### Instance variables...
		@@dpkg = '/usr/bin/dpkg'
		@@aptitude = '/usr/bin/aptitude'
		@@grep = '/bin/grep --color -i -e'
		@@gem = '/usr/bin/gem'
		@@awk = '/usr/bin/awk'

		# Check gems
		def dependences_gems(gems)
			begin
				gems.each do |dep|
					unless system("#{@@gem} list -i #{dep} >> /dev/null")
						puts "#{Tty.white}Installing #{Tty.blue} ==> #{Tty.green} #{dep} #{Tty.reset}"
						#unless system("#{@@gem} install #{dep} --no-ri --no-rdoc")
						unless system("#{@@gem} install #{dep}")
							puts "#{Tty.red} Error: Installing #{Tty.blue}==>#{Tty.red} #{dep}!#{Tty.reset}"
							exit!
						end
					end
				end	
				rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}All the gems are #{Tty.blue} ==> #{Tty.green} OK #{Tty.reset}"
		end

		# Check gems
		def dependences_deb(debs)
			begin
				puts "#{Tty.white}Updating #{Tty.blue} ==> #{Tty.green} Repositories #{Tty.reset}"
				unless system("#{@@aptitude} update >> /dev/null")
					puts "#{Tty.red} Error: Installing #{Tty.blue}==>#{Tty.red} #{dep}!#{Tty.reset}"
					exit!
				end
				puts "#{Tty.white}Repositories #{Tty.blue} ==> #{Tty.green} Updated #{Tty.reset}"

				debs.each do |dep|
						unless system("#{@@dpkg} -l | #{@@grep} #{dep} | #{@@awk} {'print $2}' | #{@@grep} \"^#{dep}$\" >> /dev/null")
						puts "#{Tty.white}Installing #{Tty.blue} ==> #{Tty.green} #{dep} #{Tty.reset}"
							unless system("#{@@aptitude} install #{dep} -y >> /dev/null")
								puts "#{Tty.red} Error: Installing #{Tty.blue}==>#{Tty.red} #{dep}!#{Tty.reset}"
								exit!
							end
						end
				end
				rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}All the deb packages are #{Tty.blue} ==> #{Tty.green} OK #{Tty.reset}"
		end

	end
	
	class Core

		### Instance variables...
		@@dpkg = '/usr/bin/dpkg'
		@@aptitude = '/usr/bin/aptitude'
		@@mknod = '/bin/mknod'
		@@grep = '/bin/grep --color -i -e'
		@@systemctl = '/bin/systemctl'
		@@awk = '/usr/bin/awk'

		### Handling block creation.
		def create_block(device,opt)
			begin
				if File.exists?(device)
					puts "#{Tty.white}Device #{Tty.blue} ==> #{Tty.white} #{device} #{Tty.blue} ==> #{Tty.green} Already Exists #{Tty.reset}"
				elsif system("#{@@mknod} #{device} #{opt}")
					puts "#{Tty.white}Created device #{Tty.blue} ==> #{Tty.green} #{device} #{Tty.reset}"
				end
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end

		end

		### Handling directory creation...
		def create_dir(dir)
			begin
				if Dir.exists?(dir)
					puts "#{Tty.white}Directory #{Tty.blue} ==> #{Tty.white} #{dir} #{Tty.blue} ==> #{Tty.green} Already Exists #{Tty.reset}"
				elsif FileUtils.mkdir_p(dir)
					puts "#{Tty.white}Created directory #{Tty.blue} ==> #{Tty.green} #{dir} #{Tty.reset}"
				end
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
		end

		### Handling move files
		def mv(src,dst)
			begin
				FileUtils.cp_r("#{src}","#{dst}", :preserve => true)
				FileUtils.rm_rf("#{src}")
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{src} #{Tty.blue} ==> #{Tty.white} moved to: #{dst} #{Tty.blue} ==> #{Tty.green} Moved #{Tty.reset}"
		end

		### Handling move files
		def cp(src,dst)
			begin
				FileUtils.cp_r("#{src}","#{dst}", :preserve => true)
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{src} #{Tty.blue} ==> #{Tty.white} copied to: #{dst} #{Tty.blue} ==> #{Tty.green} Copied #{Tty.reset}"
		end

		### Handling commands
		### TODO: checke if is verbose or not
		def exec_cmd(command)
			begin
				system("#{command} >> /dev/null")
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}Command #{Tty.blue} ==> #{Tty.white} #{command} #{Tty.blue} ==> #{Tty.green} Executed #{Tty.reset}"
		end

		### Handling move files
		def rm(path)
			begin
				FileUtils.rm_rf("#{path}")
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{path} #{Tty.blue} ==> #{Tty.green} Removed #{Tty.reset}"
		end

		### Handling move files
		def ln_s(src,dst)
			begin
				FileUtils.ln_s("#{src}","#{dst}")
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{src} #{Tty.blue} ==> #{Tty.white} linked to: #{dst} #{Tty.blue} ==> #{Tty.green} Linked #{Tty.reset}"
		end

		### Handling move files
		def ln(src,dst)
			begin
				FileUtils.ln("#{src}","#{dst}")
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{src} #{Tty.blue} ==> #{Tty.white} linked to: #{dst} #{Tty.blue} ==> #{Tty.green} Linked #{Tty.reset}"
		end

		### Handling chmod permissions
		def chmod(path,perm)
			str_perm = perm.to_s(8)
			begin
				if Dir.exists?(path) || File.exists?(path)
					if FileUtils.chmod_R(perm,"#{path}")
						puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{path} #{Tty.blue} ==> #{Tty.white} permission changed to: #{str_perm} #{Tty.blue} ==> #{Tty.green} permission Changed #{Tty.reset}"
					end
				end
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
		end

		### Handling chown permissions
		def chown(path,owner=nil,group=nil)
			begin
				if Dir.exists?(path) || File.exists?(path)
					if FileUtils.chown_R(owner, group, "#{path}")
						if group.nil?
							puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{path} #{Tty.blue} ==> #{Tty.white} changed to: owner: #{owner} #{Tty.blue} ==> #{Tty.green} Changed #{Tty.reset}"
						else
							puts "#{Tty.white}Path #{Tty.blue} ==> #{Tty.white} #{path} #{Tty.blue} ==> #{Tty.white} changed to: owner: #{owner}, group: #{group} #{Tty.blue} ==> #{Tty.green} Changed #{Tty.reset}"
						end
					end
				end
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
		end

		### Save files
		def save_file(content,path)
			begin
				file = File.new("#{path}",'w')
        		file.write content
					file.close
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			puts "#{Tty.white}File #{Tty.blue} ==> #{Tty.white} #{path} #{Tty.blue} ==> #{Tty.green} Saved #{Tty.reset}"
		end

		### Handling the service actions such as stop/start/restart
		def handle_service(service=nil,action=nil)
			service_name = service.split('.').first
			begin
				if action.nil?
					puts "#{Tty.white}Executing service #{Tty.blue} ==> #{Tty.green} #{service_name.upcase} #{Tty.reset}"
					system("#{@@systemctl} #{service}")
				else
					puts "#{Tty.white}Executing service #{Tty.blue} ==> #{Tty.white} #{service_name} #{Tty.blue} ==> #{Tty.green} #{action.upcase} #{Tty.reset}"
					system("#{@@systemctl} #{action} #{service}")
				end
			rescue Exception => e
				puts "#{Tty.red} Error: #{Tty.blue}==>#{Tty.red} #{e.message}!#{Tty.reset}"
				exit!
			end
			if action.nil?
				puts "#{Tty.white}Service #{Tty.blue} ==> #{Tty.green} #{service_name.upcase} #{Tty.reset}"
			else
				puts "#{Tty.white}Service #{Tty.blue} ==> #{Tty.white} #{service_name} #{Tty.blue} ==> #{Tty.green} #{action.upcase} #{Tty.reset}"
			end
		end

		### Handle configuration files
		def load_conf(path=nil)
			parsed = begin
		  YAML.load(File.open("#{path}"))
			rescue Exception => e
			  puts "Could not parse YAML: #{e.message}"
			end
			return to_ostruct(parsed)
		end

		def to_ostruct(object)
		  case object
		  when Hash
		    OpenStruct.new(Hash[object.map {|k, v| [k, to_ostruct(v)] }])
		  when Array
		    object.map {|x| to_ostruct(x) }
		  else
		    object
		  end
		end

	end
end
